/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.dialog;

import static org.eclipse.jface.dialogs.IDialogConstants.CANCEL_ID;
import static org.eclipse.jface.dialogs.IDialogConstants.OK_ID;
import static org.eclipse.jface.dialogs.IDialogConstants.OK_LABEL;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.wb.swt.ResourceManager;

/**
 * This dialog offers a common dialog to show multi-root tree structures and
 * allows user to select one item in the tree. It also offers interface to edit
 * the tree on the fly.
 * 
 * TODO shouldn't we move this Dialog to org.fortiss.tooling.base.ui?
 * 
 * @author mou
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D56739A3E2352F3C5BE90A37CB7F40F9
 */
public class ElementTreeSingleSelectDialog extends TitleAreaDialog {

	/** GUI Factory */
	private final FormToolkit formToolkit = new FormToolkit(
			Display.getDefault());

	/** the editing module */
	private IDialogEditSupport dialogEditSupport;

	/** the root elements */
	private ArrayList<Object> rootElements;

	/** the selected element as tree path */
	private TreePath selectedPath;

	/** the content provider */
	private ITreeContentProvider contentSupport;

	/** the label provider */
	private ILabelProvider labelSupport;

	/** the main tree viewer */
	private TreeViewer elementTreeViewer;

	/**
	 * Default Constructor for GUI builder.
	 * 
	 * @param parentShell
	 * 
	 * @deprecated
	 * @wbp.parser.constructor
	 */
	@Deprecated
	public ElementTreeSingleSelectDialog(Shell parentShell) {
		super(parentShell);
		setShellStyle(SWT.SHELL_TRIM);
		setErrorMessage("");
		setHelpAvailable(false);
	}

	/**
	 * Main constructor
	 * 
	 * @param parentShell
	 *            the parent shell
	 * @param roots
	 *            the root elements
	 * @param selected
	 *            the pre-selected element or null
	 * @param contentProvider
	 *            the content provider
	 * @param labelProvider
	 *            the label provider
	 * @param edit
	 *            the editing module or null
	 */
	public ElementTreeSingleSelectDialog(Shell parentShell, List<?> roots,
			Object selected, ITreeContentProvider contentProvider,
			ILabelProvider labelProvider, IDialogEditSupport edit) {
		super(parentShell);

		if (roots == null) {
			rootElements = new ArrayList<Object>();
		} else {
			rootElements = new ArrayList<Object>(roots);
		}

		if (selected == null) {
			selectedPath = null;
		} else {
			selectedPath = computeTreePath(rootElements, selected);
		}

		dialogEditSupport = edit;
		contentSupport = contentProvider;
		labelSupport = labelProvider;
	}

	/** {@inheritDoc} */
	@Override
	protected Control createDialogArea(Composite parent) {
		// CAUTION: The code below is automatically generated, do not edit
		// manually!

		setHelpAvailable(false);

		setTitle("Select");
		setMessage("Select one or more elements");

		// create controls
		Composite area = (Composite) super.createDialogArea(parent);
		Composite container = new Composite(area, SWT.NONE);
		GridLayout gl_container = new GridLayout(3, false);

		gl_container.verticalSpacing = 2;
		gl_container.marginRight = 5;
		gl_container.marginLeft = 5;
		gl_container.marginBottom = 5;
		gl_container.marginTop = 5;
		gl_container.marginWidth = 0;
		gl_container.marginHeight = 0;
		container.setLayout(gl_container);
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite lefttoolbar = new Composite(container, SWT.NONE);
		lefttoolbar.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false,
				true, 1, 1));
		GridLayout gl_lefttoolbar = new GridLayout(1, false);
		gl_lefttoolbar.verticalSpacing = 2;
		gl_lefttoolbar.horizontalSpacing = 2;
		gl_lefttoolbar.marginHeight = 0;
		gl_lefttoolbar.marginWidth = 0;
		lefttoolbar.setLayout(gl_lefttoolbar);

		Button createButton = new Button(lefttoolbar, SWT.NONE);
		createButton.setImage(ResourceManager.getPluginImage("org.eclipse.ui",
				"/icons/full/obj16/add_obj.gif"));
		formToolkit.adapt(createButton, true, true);

		Button deleteButton = new Button(lefttoolbar, SWT.NONE);
		deleteButton.setImage(ResourceManager.getPluginImage("org.eclipse.ui",
				"/icons/full/obj16/delete_obj.gif"));
		formToolkit.adapt(deleteButton, true, true);

		Button editButton = new Button(lefttoolbar, SWT.NONE);
		editButton.setImage(ResourceManager.getPluginImage("org.eclipse.ui",
				"/icons/full/etool16/editor_area.gif"));
		formToolkit.adapt(editButton, true, true);

		elementTreeViewer = new TreeViewer(container, SWT.BORDER);
		Tree elementTree = elementTreeViewer.getTree();
		elementTree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true,
				1, 1));

		Composite toolbarbox = new Composite(container, SWT.NONE);
		GridLayout gl_toolbarbox = new GridLayout(1, false);
		gl_toolbarbox.verticalSpacing = 2;
		gl_toolbarbox.horizontalSpacing = 2;
		gl_toolbarbox.marginHeight = 0;
		gl_toolbarbox.marginWidth = 0;
		toolbarbox.setLayout(gl_toolbarbox);
		toolbarbox.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false,
				true, 1, 1));

		// create listeners
		createButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				createElement();
			}
		});

		deleteButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				deleteElement();
			}
		});

		editButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				editElement();
			}
		});

		elementTreeViewer
				.addSelectionChangedListener(new ISelectionChangedListener() {
					@Override
					public void selectionChanged(SelectionChangedEvent event) {
						selectElement();
					}
				});

		// label, content and input
		elementTreeViewer.setLabelProvider(labelSupport);

		elementTreeViewer.setContentProvider(new MultiRootTreeContentProvider(
				rootElements, contentSupport));

		elementTreeViewer.setInput(rootElements);

		if (selectedPath != null)
			elementTreeViewer.setSelection(new TreeSelection(selectedPath),
					true);

		// hide edit toolbar if no edit function is available
		if (dialogEditSupport == null) {
			lefttoolbar.setVisible(false);
			((GridData) lefttoolbar.getLayoutData()).exclude = true;
		}

		return area;
	}

	/** {@inheritDoc} */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, OK_ID, OK_LABEL, false);
		createButton(parent, CANCEL_ID, "Cancel", true);
	}

	/** {@inheritDoc} */
	@Override
	protected Point getInitialSize() {
		return new Point(480, 480);
	}

	/** Returns configuration. */
	public IDialogEditSupport getEditSupport() {
		return dialogEditSupport;
	}

	/** Returns rootElements. */
	public List<Object> getRootElements() {
		return rootElements;
	}

	/** Returns selectedElements. */
	public Object getSelectedElement() {
		if (selectedPath == null)
			return null;
		return selectedPath.getLastSegment();
	}

	/** Returns contentSupport. */
	public IStructuredContentProvider getContentProvider() {
		return contentSupport;
	}

	/** Returns labelSupport. */
	public ILabelProvider getLabelProvider() {
		return labelSupport;
	}

	/** Create an element */
	private void createElement() {
		if (dialogEditSupport == null)
			return;

		try {

			// find the path to hold the new element
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() != null
					&& selection.getPaths().length != 1)
				return; // at most 1 element can be selected

			// create the new element
			Object o;
			if (selection.getPaths() == null) {
				o = dialogEditSupport.createElement(null);
			} else {
				o = dialogEditSupport.createElement(selection.getPaths()[0]
						.getLastSegment());
			}
			if (o == null)
				return;

			TreePath path;
			if (selection.getPaths() == null) {
				path = new TreePath(new Object[] { rootElements });
				rootElements.add(o);
			} else {
				path = selection.getPaths()[0];
			}

			// insert the new element
			elementTreeViewer.add(path, o);
			elementTreeViewer.reveal(o);
		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/** Call delete function on the selected elements */
	private void deleteElement() {
		if (dialogEditSupport == null)
			return;

		try {
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() == null)
				return;

			// delete selected elements
			for (TreePath path : selection.getPaths()) {
				if (dialogEditSupport.deleteElement(path.getLastSegment())) {
					if (rootElements.contains(path.getLastSegment()))
						rootElements.remove(path.getLastSegment());

					elementTreeViewer.remove(path);

					if (path.equals(selectedPath)) {
						selectedPath = null;
					}
				}
			}
		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/** Call edit function on the selected element */
	private void editElement() {
		if (dialogEditSupport == null)
			return;

		try {
			// find the path to the element
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() == null
					|| selection.getPaths().length != 1)
				return; // 1 element must be selected

			TreePath path = selection.getPaths()[0];
			dialogEditSupport.editElement(path.getLastSegment());

			elementTreeViewer.refresh();
		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/** update the selected element by its tree path */
	private void selectElement() {
		// find the path to the element
		ITreeSelection selection = (ITreeSelection) elementTreeViewer
				.getSelection();
		if (selection.getPaths() == null || selection.getPaths().length != 1) {
			selectedPath = null;
			getButton(OK_ID).setEnabled(false);
		} else {
			selectedPath = selection.getPaths()[0];
			getButton(OK_ID).setEnabled(true);
		}
	}

	/**
	 * Create the tree path from one of the given root elements to the child
	 * element. The content provider will be used to compute the direct parent
	 * of the given child element.
	 * 
	 * @param roots
	 *            all possible parent elements
	 * @param leaf
	 *            the child element
	 * @return a tree path starting from one of the root elements and up to the
	 *         child element, or null if no such path exists
	 */
	private TreePath computeTreePath(List<?> roots, Object leaf) {
		LinkedList<Object> path = new LinkedList<Object>();

		Object parent = leaf;
		while (!roots.contains(parent)) {
			path.addFirst(parent);
			parent = contentSupport.getParent(parent);

			if (parent == null)
				return null;
		}
		path.addFirst(parent);

		return new TreePath(path.toArray());
	}
}