/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.command;

import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.changeAllConnectionPoints;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.ConnectionEditPartBase;
import org.fortiss.tooling.base.ui.editpart.ConnectorEditPartBase;
import org.fortiss.tooling.base.ui.editpart.ElementEditPartBase;
import org.fortiss.tooling.base.ui.editpart.FreeConnectorEditPartBase;

/**
 * A command to move the elements and free connectors within a diagram.
 * 
 * @author wandinger
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 54B9F493565F91469FF3EE1CF7BBD74C
 */
public class MoveWithinDiagramCommand extends ChangeBoundsRequestCommandBase {

	/** Constructor. */
	public MoveWithinDiagramCommand(ChangeBoundsRequest request) {
		super(request);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public void execute() {
		bendpointGroupMove(request.getEditParts());
		for (EditPart editpart : (List<EditPart>) getRequest().getEditParts()) {
			if (editpart instanceof ElementEditPartBase) {
				ElementEditPartBase<? extends ILayoutedModelElement> elementEditPart = (ElementEditPartBase<? extends ILayoutedModelElement>) editpart;
				elementEditPart.getLayoutConfiguration()
						.performElementMoveAndResize(elementEditPart, request);
			} else if (editpart instanceof FreeConnectorEditPartBase) {
				FreeConnectorEditPartBase<? extends ILayoutedModelElement> connectorEditPart = (FreeConnectorEditPartBase<? extends ILayoutedModelElement>) editpart;
				connectorEditPart.getLayoutConfiguration()
						.performFreeConnectorMove(connectorEditPart, request);
			}
		}
	}

	/** Move bend points along with a group of selected elements */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void bendpointGroupMove(List<EditPart> parts) {
		// TODO (FH): https://af3.fortiss.org/issues/376
		// This has to be reworked to delegate to diagram configuration
		for (EditPart part : parts) {
			// cover also sticky connectors
			if (part.getChildren().size() > 0) {
				bendpointGroupMove(part.getChildren());
			}
			// filter non connective edit parts
			if (!(part instanceof ConnectorEditPartBase)) {
				break;
			}

			ConnectorEditPartBase connector = (ConnectorEditPartBase) part;

			for (ConnectionEditPartBase connection : (List<ConnectionEditPartBase>) connector
					.getSourceConnections()) {
				// IF two states where selected, OR IF the 'connection' is a
				// self-transition, THEN move also the bendpoints of the
				// connections between these states.
				if ((connection.getTarget().getSelected() != EditPart.SELECTED_NONE)
						|| (connection.getTarget().getParent().getSelected() != EditPart.SELECTED_NONE)) {
					changeAllConnectionPoints(
							(ILayoutedModelElement) connection.getModel(),
							request.getMoveDelta().x, request.getMoveDelta().y);
				}

			}
		}
	}
}
