/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.command.bendpoint;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.layout.IDiagramLayoutConfiguration;

/**
 * Abstract command related to bend points.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: C20AF7D4BE6A76ABC8C3D7718CBC4B3C
 */
public abstract class CommandBase extends Command {

	/** The position where the point should be placed. */
	protected Point location;

	/** The index of the bend point */
	protected int index;

	/** The connection where the bend point is added. */
	protected ILayoutedModelElement connection;

	/** The {@link IDiagramLayoutConfiguration}. */
	protected final IDiagramLayoutConfiguration layouter;

	/** Creates a bend point command. */
	public CommandBase(ILayoutedModelElement connection, int index,
			Point location, IDiagramLayoutConfiguration layouter) {
		this.connection = connection;
		this.index = index;
		this.location = location;
		this.layouter = layouter;
	}

	/** Performs layout using the {@link #layouter}. */
	protected final void doLayout() {
		if (layouter != null) {
			layouter.performBendPointLayout(location);
		}
	}
}
