/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.figure;

import org.eclipse.jface.preference.IPreferenceStore;
import org.fortiss.tooling.base.ui.ToolingBaseUIActivator;

/**
 * The visual style that {@link PrettyInsetEllipse} and
 * {@link PrettyRoundedRectangle} use for representation.
 * 
 * @author schwitzer
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: A56EAF879DF84E2265CA1617D0E8C55F
 */
public enum EVisualStyle {
	/** Traditional (since 2007) CCTS style with soft gradients. */
	Traditional,

	/** Glossy (since 2010) CCTS style with glossy reflective gradients. */
	Glossy;

	/** Constant used as key for the preference-store of this plug-in. */
	public static final String ID = "figures.visualStyle";

	/** Cached preference-store to lookup visual style. */
	private static IPreferenceStore preferenceStore = ToolingBaseUIActivator
			.getDefault().getPreferenceStore();

	/**
	 * Returns the visual style from the preference store of this plug-in's
	 * {@link ToolingBaseUIActivator}.
	 */
	public static EVisualStyle get() {
		final String name = preferenceStore.getString(ID);
		if (name == null || name.length() == 0) {
			return Traditional; // Default.
		}
		return valueOf(name);
	}

	/**
	 * Sets the visual style in the preference store of this plug-in's
	 * {@link ToolingBaseUIActivator}.
	 */
	public static void set(EVisualStyle style) {
		preferenceStore.setValue(ID, style.name());
	}
}
