/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.utils;

import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;

/**
 * Utility code to support zooming.
 * 
 * @author hummel
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 57E5F4C587CD015E3B85AC0666F7BA44
 */
public class ZoomUtils {

	/** Extracts the current zoom level from the given root edit part. */
	public static double determineZoom(RootEditPart rootEditPart) {
		if (rootEditPart instanceof ScalableRootEditPart) {
			return ((ScalableRootEditPart) rootEditPart).getZoomManager()
					.getZoom();
		}
		if (rootEditPart instanceof ScalableFreeformRootEditPart) {
			return ((ScalableFreeformRootEditPart) rootEditPart)
					.getZoomManager().getZoom();
		}
		return 1;
	}

	/**
	 * Fixes a {@link ChangeBoundsRequest} for the current zoom level. Both the
	 * move and size delta are changed.
	 */
	public static void fixChangeBoundRequest(ChangeBoundsRequest request,
			double zoom) {
		if (zoom <= 0 || zoom == 1) {
			return;
		}
		request.getMoveDelta().scale(1 / zoom);
		request.getSizeDelta().scale(1 / zoom);
	}
}
