/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.utils;

import static org.fortiss.tooling.base.utils.LayoutModelElementFactory.createAngle;

import org.fortiss.tooling.base.model.layout.Angle;
import org.fortiss.tooling.base.model.layout.ILayoutData;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;

/**
 * Utility methods for {@link Angle} layout data.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: DE79B340A0B6E270C967987E242746CC
 */
public final class AngleUtils {

	/**
	 * Returns an angle.
	 * 
	 * @param layouted
	 *            the layouted model element
	 * @param key
	 *            the layout key
	 * @return the angle stored under the layout key
	 */
	public static Angle getAngle(ILayoutedModelElement layouted, String key) {
		for (ILayoutData ld : layouted.getLayoutDataList()) {
			if (ld.getKey().equals(key) && ld instanceof Angle) {
				return (Angle) ld;
			}
		}
		return null;
	}

	/**
	 * Sets a {@link Angle} layout data.
	 * 
	 * @param layouted
	 *            the layouted model element
	 * @param key
	 *            the layout key
	 * @param angle
	 *            the angle value
	 */
	public static void setAngle(ILayoutedModelElement layouted, String key,
			double angle) {
		Angle old = getAngle(layouted, key);
		if (old != null) {
			layouted.getLayoutDataList().remove(old);
		}

		Angle a = createAngle(angle, key);
		layouted.getLayoutDataList().add(a);
	}
}
