/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.dnd;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.widgets.Widget;
import org.fortiss.tooling.kernel.service.IElementCompositorService;

/**
 * {@link DropTargetAdapter} for completing an EObject composition via a drop
 * gesture. If the {@link IElementCompositorService} allows the composition the
 * {@link DND#DROP_COPY} detail is set as user feedback.
 * <p>
 * This implementation uses the raw SWT DND mechanism.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: C26BA94BBBB077835EDB9298D41E7EF5
 */
public abstract class ElementCompositionDropTargetAdapter extends
		DropTargetAdapter {

	/** {@inheritDoc} */
	@Override
	public void drop(DropTargetEvent event) {
		ElementCompositionSource dragSource = getDragSourceFromTransfer();
		EObject target = getTargetModel(event.item);
		IElementCompositorService.INSTANCE.compose(target,
				dragSource.getDesignatedElement(), null);
	}

	/** {@inheritDoc} */
	@Override
	public void dragOver(DropTargetEvent event) {
		ElementCompositionSource dragSource = getDragSourceFromTransfer();
		EObject target = getTargetModel(event.item);
		if (target == null || dragSource.getDesignatedElement() == null) {
			event.detail = DND.DROP_NONE;
			return;
		}
		if (IElementCompositorService.INSTANCE.canCompose(target,
				dragSource.getDesignatedElement(), null)) {
			event.detail = DND.DROP_COPY;
			return;
		}
		event.detail = DND.DROP_NONE;
	}

	/** Returns the current target element. */
	protected abstract EObject getTargetModel(Widget dropTarget);

	/** Returns the transfered object. */
	private ElementCompositionSource getDragSourceFromTransfer() {
		return (ElementCompositionSource) CompositionServiceLocalDNDTransfer
				.getInstance().getObject();
	}
}
