/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import org.eclipse.core.resources.IFile;
import org.fortiss.tooling.kernel.service.IEclipseResourceStorageService;

/**
 * Interface for providing Eclipse workspace resources as model files to the
 * kernel. This interface is usually implemented by a class of the final tool
 * product, e.g. the AF3 tool product uses <I>*.af3</I> as its model files.
 * <P>
 * Extensions are handled by the {@link IEclipseResourceStorageService}.
 * <P>
 * The persistency storage provider, which uses Eclipse resources, does not know
 * which files should be considered model files. Therefore, a concrete product
 * built above the kernel is required to decide whether the file provided with
 * {@link #isStorageLocation(IFile)} should be considered.
 * 
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: CE0537EAE8D9116253F7769DF9378FDE
 */
public interface IEclipseResourceStorageLocationProvider {

	/**
	 * Returns whether the given {@link IFile} should be considered as a model
	 * file.
	 */
	boolean isStorageLocation(IFile file);
}
