/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2005-2010 Technische Universitaet Muenchen                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;

/**
 * Utility code for performing logging.
 * 
 * Note: This class originates form the org.conqat.ide.commons.ui
 * implementation. It re-packaged with the kernel to avoid any dependency to
 * eclipse.ui packages. See http://www.conqat.org for the original version.
 * 
 * @author hummel
 * @author schwitzer
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: E04BEF36D0F45F63BE30A59EEE54F6A4
 */
public class LoggingUtils {

	/**
	 * Logs a message that occurred in the context of the given plug-in with the
	 * given severity and a given {@link Throwable} that originally caused this
	 * log message.
	 */
	private static void log(Plugin plugin, String message, int severity,
			Throwable cause) {
		String pluginId = plugin.getBundle().getSymbolicName();
		IStatus status = new Status(severity, pluginId, message, cause);
		plugin.getLog().log(status);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#ERROR}.
	 * 
	 * @param plugin
	 *            the originating plugin
	 * @param message
	 *            the error message
	 * @param cause
	 *            the causing exception
	 */
	public static void error(Plugin plugin, String message, Throwable cause) {
		log(plugin, message, IStatus.ERROR, cause);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#ERROR}.
	 * 
	 * @param plugin
	 *            the originating plugin
	 * @param message
	 *            the error message
	 */
	public static void error(Plugin plugin, String message) {
		error(plugin, message, null);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#WARNING}.
	 * 
	 * @param plugin
	 *            the originating plugin
	 * @param message
	 *            the warning message
	 * @param cause
	 *            the causing exception
	 */
	public static void warning(Plugin plugin, String message, Throwable cause) {
		log(plugin, message, IStatus.WARNING, cause);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#WARNING}.
	 * 
	 * @param plugin
	 *            the originating plugin
	 * @param message
	 *            the warning message
	 */
	public static void warning(Plugin plugin, String message) {
		warning(plugin, message, null);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#INFO}.
	 * 
	 * @param plugin
	 *            the originating plugin
	 * @param message
	 *            the information message
	 * @param cause
	 *            the causing exception
	 */
	public static void info(Plugin plugin, String message, Throwable cause) {
		log(plugin, message, IStatus.INFO, cause);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#INFO}.
	 * 
	 * @param plugin
	 *            the originating plugin
	 * @param message
	 *            the information message
	 */
	public static void info(Plugin plugin, String message) {
		info(plugin, message, null);
	}
}
